@@ -1,7 +1,6 @@
 # -*- coding: utf-8 -*-
 """Create direct, total and mediation Graphviz graph from dot_str"""
 import re
-import locale
 import subprocess
 from typing import Iterable
 from itertools import chain
@@ -12,6 +11,7 @@ import numpy as np
 import networkx
 
 from causing.model import Model
+from causing import utils
 
 
 DOT_COMMAND = "dot"
@@ -78,10 +78,12 @@ def save_graph(path: Path, graph_dot):
 def annotated_graphs(
     m: Model,
     graph_json,
-    ids: Iterable[str],
+    ids: Iterable[str] = None,
     node_labels: dict[str, str] = {},
 ) -> Iterable[networkx.DiGraph]:
     """Return DiGraphs with all information required to draw IME graphs"""
+    if ids is None:
+        ids = [str(i + 1) for i in range(len(graph_json["eyx_indivs"]))]
     for graph_id, exj, eyj, eyx, eyy in zip(
         ids,
         np.array(graph_json["exj_indivs"]).T,
@@ -129,29 +131,41 @@ def color(val, max_val, palette):
     return palette[clipped_ind]
 
 
-def graph_to_dot(g: networkx.DiGraph, invisible_edges):
-    dot_str = """digraph {
+GRAPH_OPTIONS_STR = """
     node [style="filled,rounded"]
     node [shape=box]
     node [color="#444444"]
     ratio="compress"
     size="8,10"
 """
+
+
+def graph_to_dot(
+    g: networkx.DiGraph,
+    invisible_edges={},
+    node_palette=NODE_PALETTE,
+    edge_palette=EDGE_PALETTE,
+    pen_width_palette=PEN_WIDTH_PALETTE,
+    graph_options_str=GRAPH_OPTIONS_STR,
+    in_percent=False,
+    min_sig_figures=3,
+):
+    dot_str = "digraph {" + graph_options_str
     max_val = max(
         [abs(data["effect"]) for _, data in g.nodes(data=True)]
         + [abs(data["effect"]) for _, _, data in g.edges(data=True)]
     )
 
     for node, data in g.nodes(data=True):
-        eff_str = locale.format_string("%.2f%%", data["effect"] * 100)
-        label = data["label"].replace("\n", r"\n") + r"\n" + eff_str
-        col_str = color(data["effect"], max_val, palette=NODE_PALETTE)
+        eff_str = utils.fmt_min_sig(data["effect"], min_sig_figures, percent=in_percent)
+        label = data.get("label", node).replace("\n", r"\n") + r"\n" + eff_str
+        col_str = color(data["effect"], max_val, palette=node_palette)
         dot_str += f'    "{node}"[label = "{label}" fillcolor="{col_str}"]\n'
 
     for from_node, to_node, data in g.edges(data=True):
-        eff_str = locale.format_string("%.2f%%", data["effect"] * 100)
-        col_str = color(data["effect"], max_val, palette=EDGE_PALETTE)
-        penwidth = color(data["effect"], max_val, palette=PEN_WIDTH_PALETTE)
+        eff_str = utils.fmt_min_sig(data["effect"], min_sig_figures, percent=in_percent)
+        col_str = color(data["effect"], max_val, palette=edge_palette)
+        penwidth = color(data["effect"], max_val, palette=pen_width_palette)
         dot_str += f'    "{from_node}" -> "{to_node}" [label="{eff_str}" color="{col_str}" penwidth="{penwidth}"]\n'
 
     for from_node, to_node in invisible_edges:
@@ -161,11 +175,82 @@ def graph_to_dot(g: networkx.DiGraph, invisible_edges):
     return dot_str
 
 
-def create_graphs(
-    graphs: Iterable[networkx.DiGraph], output_dir: Path, invisible_edges=()
-):
+def create_graphs(graphs: Iterable[networkx.DiGraph], output_dir: Path, **kwargs):
     for g in graphs:
         filename = f"IME_{g.graph['id']}.svg"
         print("Create", filename)
-        dot_str = graph_to_dot(g, invisible_edges)
+        dot_str = graph_to_dot(g, **kwargs)
         save_graph(output_dir / filename, dot_str)
+
+
+def remove_node_keep_edges(graph, node):
+    """Keep transitive connections when removing node.
+
+    Removing B from A->B->C will result in A->C.
+
+    WARNING: This function only approximates the edge effects. To get accurate
+    results, you must shrink the model and recalculate the effects, instead.
+    """
+
+    total_out_effect = sum(
+        out_data["effect"] for _, _, out_data in graph.out_edges(node, data=True)
+    )
+    num_out_edges = len(graph.out_edges(node))
+    for a, _, in_data in graph.in_edges(node, data=True):
+        for _, b, out_data in graph.out_edges(node, data=True):
+            if total_out_effect == 0:
+                # If all outgoing edges have no effect, distribute the incoming effects
+                # evenly across all outgoing edges.
+                new_edge_effect = in_data["effect"] / num_out_edges
+            else:
+                new_edge_effect = (
+                    in_data["effect"] * out_data["effect"] / total_out_effect
+                )
+            if graph.has_edge(a, b):
+                graph[a][b]["effect"] += new_edge_effect
+            else:
+                graph.add_edge(a, b, effect=new_edge_effect)
+
+    graph.remove_node(node)
+
+
+def recalc_graphs(graphs, model, xdat) -> Iterable[networkx.DiGraph]:
+    """Recalculate node and edge effects in graph.
+
+    Do this after modifying the graphs (typically with `remove_node_keep_edges`)
+    to calculate exact effects.
+    `graphs` must be in the format generated by `annotated_graphs` and in the
+    same order as individuals within `xdat`.
+    """
+    yhat = model.compute(xdat)
+    yhat_mean = np.mean(yhat, axis=1)
+    xdat_mean = np.mean(xdat, axis=1)
+
+    for i, approx_graph in enumerate(graphs):
+        individual_xdat = xdat[:, i : i + 1]
+        removed_nodes = set(model.graph.nodes) - set(approx_graph.nodes)
+
+        # Calc effects on shrunken model
+        individual_model = model.shrink(removed_nodes)
+        effects = individual_model.calc_effects(
+            individual_xdat,
+            xdat_mean=xdat_mean,
+            yhat_mean=yhat_mean[[yvar not in removed_nodes for yvar in model.yvars]],
+        )
+
+        # Get graph for shrunken model
+        [g] = annotated_graphs(
+            individual_model,
+            effects,
+            node_labels={
+                n: data["label"]
+                for n, data in approx_graph.nodes(data=True)
+                if "label" in data
+            },
+        )
+        for xvar in set(model.xvars) & removed_nodes & g.nodes():
+            g.remove_node(xvar)
+
+        # Preserve graph attributes
+        g.graph = approx_graph.graph
+        yield g
